@@ -138,7 +138,7 @@ class JSONAPIResponse(object):
         self.status_code = 200
         self.data = {
             'jsonapi': {'version': '1.0'},
-            'meta': {'sqlalchemy_jsonapi_version': '3.0.1'}
+            'meta': {'sqlalchemy_jsonapi_version': '3.0.2'}
         }
 
 
@@ -222,7 +222,7 @@ class JSONAPI(object):
             if name.startswith('_'):
                 continue
 
-            prepped_name = underscore(pluralize(name))
+            prepped_name = self._api_type_for_model(model)
             api_type = getattr(model, '__jsonapi_type_override__', prepped_name)
 
             model.__jsonapi_attribute_descriptors__ = {}
@@ -272,6 +272,10 @@ class JSONAPI(object):
                             perm_idv[check_perm] = prop_value
             self.models[model.__jsonapi_type__] = model
 
+    def _api_type_for_model(self, model):
+        return underscore(pluralize(model.__name__))
+
+
     def _fetch_model(self, api_type):
         if api_type not in self.models.keys():
             raise ResourceTypeNotFoundError(api_type)
@@ -374,7 +378,7 @@ class JSONAPI(object):
                                                            new_include, fields)
                         included = built.pop('included')
                         to_ret['included'].update(included)
-                        to_ret['included'][(api_type, related.id)] = built
+                        to_ret['included'][(related.__jsonapi_type__, related.id)] = built
 
             else:
                 if key in local_fields:
@@ -398,7 +402,7 @@ class JSONAPI(object):
                                                            fields)
                         included = built.pop('included')
                         to_ret['included'].update(included)
-                        to_ret['included'][(api_type, item.id)] = built
+                        to_ret['included'][(item.__jsonapi_type__, item.id)] = built
 
         for key in set(orm_desc_keys) - attrs_to_ignore:
             try:
@@ -837,7 +841,7 @@ class JSONAPI(object):
         if not ({'type', 'id'} <= set(json_data['data'].keys())):
             raise BadRequestError('Missing type or id')
 
-        if json_data['data']['id'] != str(resource.id):
+        if str(json_data['data']['id']) != str(resource.id):
             raise BadRequestError('IDs do not match')
 
         if json_data['data']['type'] != resource.__jsonapi_type__:
@@ -1004,12 +1008,14 @@ class JSONAPI(object):
                         ', '.join(list(data_keys -
                                        model_keys)), model.__jsonapi_type__))
 
-            for setter, value in setters:
-                setter(resource, value)
+            with session.no_autoflush:
+                for setter, value in setters:
+                    setter(resource, value)
+
+                for key in data_keys:
+                    setter = get_attr_desc(resource, key, AttributeActions.SET)
+                    setter(resource, data['data']['attributes'][key])
 
-            for key in data_keys & model_keys:
-                setter = get_attr_desc(resource, key, AttributeActions.SET)
-                setter(resource, data['data']['attributes'][key])
             session.add(resource)
             session.commit()
         except IntegrityError as e:
